#define SP_TOOLBOX_C

#include <gnome.h>
#include <glade/glade.h>
#include "sodipodi-private.h"
#include "toolbox.h"
#include "sodipodi.h"
#include "event-broker.h"
#include <libgnomeui/gnome-pixmap.h>
#include <libgnomeui/gnome-stock.h>
#include "dialogs/transformation.h"
#include "zoom-context.h"
#include "selection.h"
#include "sp-item-transform.h"
#include "desktop-handles.h"

// needed for draw toolbox update
#include "select-context.h"
#include "node-context.h"
#include "rect-context.h"
#include "ellipse-context.h"
#include "draw-context.h"
#include "text-context.h"
#include "zoom-context.h"


static GladeXML  * main_toolbox_xml = NULL;
static GtkWidget * main_toolbox_dialog = NULL;
static GtkWidget * main_vbox = NULL;
static GladeXML  * quit_xml = NULL;
static GtkWidget * quit_dialog = NULL;

static GtkWidget * draw_select = NULL;
static GtkWidget * draw_node = NULL;
static GtkWidget * draw_zoom = NULL;
static GtkWidget * draw_text = NULL;
static GtkWidget * draw_rect = NULL;
static GtkWidget * draw_ellipse = NULL;
static GtkWidget * draw_freehand = NULL;

static GtkWidget * flip_button = NULL;
static GtkWidget * fh_pixmap = NULL ;
static GtkWidget * fv_pixmap = NULL;
GtkWidget * zoom_any = NULL;

typedef enum {
  FLIP_HOR,
  FLIP_VER,
} SPObjectFlipMode;

SPObjectFlipMode object_flip_mode = FLIP_HOR;

typedef struct _SPToolBox SPToolBox;
struct _SPToolBox {
  gboolean visible;     
  gboolean standalone ;
  gboolean init ;
  gboolean inmain ;
  char* dialog_name;
  char* icon;
  GtkWidget * MainVBox;
  GtkWidget * DialogVBox;
  GtkWidget * Dialog;
  GtkWidget * BoxTable;
  GtkWidget * Hide;
  GtkWidget * Seperate;
  GtkWidget * Close;
  GladeXML * DialogXML;
};

void sp_toolbox_init (SPToolBox * toolbox);
void initialize_toolbox (SPToolBox * toolbox);
void toolbox_toggle_main (GtkWidget * widget, SPToolBox * toolbox);
void toolbox_close (GtkWidget * widget, SPToolBox * toolbox);
void maintoolbox_destroy (GtkWidget * widget, SPToolBox * toolbox);
void toolbox_delete (GtkWidget * widget, SPToolBox * toolbox);
void toolbox_toggle_seperate (GtkWidget * widget, SPToolBox * toolbox);
//void sp_select_draw_mode (GtkWidget * widget);
void quit_yep (void);
void quit_nope (void);
void sp_toolbox_create_widgets (SPToolBox * toolbox);
void sp_toolbox_expose (SPToolBox * toolbox);
//void create_draw_box (void);
void object_flip (GtkWidget * widget, GdkEventButton * event);
void object_rotate_90 (void);

static void sp_update_draw_toolbox (Sodipodi * sodipodi, SPEventContext * eventcontext, gpointer data);

// our toolboxen
static SPToolBox draw_box, zoom_box, node_box, select_box, file_box, edit_box, object_box;


void
sp_maintoolbox_create (void)
{
	if (main_toolbox_dialog == NULL) {
	        // main toolbox
		main_toolbox_xml = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "maintoolbox");
		if (main_toolbox_xml == NULL) return;
		glade_xml_signal_autoconnect (main_toolbox_xml);
		main_toolbox_dialog = glade_xml_get_widget (main_toolbox_xml, "maintoolbox");
		if (main_toolbox_dialog == NULL) return;

		sodipodi_ref ();

		main_vbox = glade_xml_get_widget (main_toolbox_xml, "main_vbox");

		// file toolbox
		file_box.DialogXML = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "file_table");
       		glade_xml_signal_autoconnect (file_box.DialogXML);

		file_box.visible = TRUE;
		file_box.standalone = FALSE;
		file_box.inmain = TRUE;
		file_box.dialog_name = " File ";
		file_box.icon = SODIPODI_GLADEDIR "/toolbox_file.xpm";

		sp_toolbox_create_widgets (&file_box);
		file_box.BoxTable = glade_xml_get_widget (file_box.DialogXML, "file_table");
		sp_toolbox_expose (&file_box);

		// edit toolbox
		edit_box.DialogXML = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "edit_table");
       		glade_xml_signal_autoconnect (edit_box.DialogXML);

		edit_box.visible = TRUE;
		edit_box.standalone = FALSE;
		edit_box.inmain = TRUE;
		edit_box.dialog_name = " Edit ";
		edit_box.icon = SODIPODI_GLADEDIR "/toolbox_edit.xpm";

		sp_toolbox_create_widgets (&edit_box);
		edit_box.BoxTable = glade_xml_get_widget (edit_box.DialogXML, "edit_table");
		sp_toolbox_expose (&edit_box);

		// objects toolbox
		object_box.DialogXML = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "object_table");
		glade_xml_signal_autoconnect (object_box.DialogXML);

		object_box.visible = TRUE;
		object_box.standalone = FALSE;
		object_box.inmain = TRUE;
		object_box.dialog_name = " Object ";
		object_box.icon = SODIPODI_GLADEDIR "/toolbox_object.xpm";

		flip_button = glade_xml_get_widget (object_box.DialogXML, "flip_button");
		fh_pixmap = glade_xml_get_widget (object_box.DialogXML, "fh_pixmap");
		fv_pixmap = glade_xml_get_widget (object_box.DialogXML, "fv_pixmap");
		object_flip_mode = FLIP_HOR;
		gtk_widget_show (fh_pixmap);

		sp_toolbox_create_widgets (&object_box);
		object_box.BoxTable = glade_xml_get_widget (object_box.DialogXML, "object_table");
		sp_toolbox_expose (&object_box);
		// select toolbox
		select_box.DialogXML = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "select_table");
		glade_xml_signal_autoconnect (select_box.DialogXML);

		select_box.visible = FALSE;
		select_box.standalone = FALSE;
		select_box.inmain = TRUE;
		select_box.dialog_name = " Selection ";
		select_box.icon = SODIPODI_GLADEDIR "/toolbox_select.xpm";

		sp_toolbox_create_widgets (&select_box);
		select_box.BoxTable = glade_xml_get_widget (select_box.DialogXML, "select_table");
		sp_toolbox_expose (&select_box);
		//draw_box
		draw_box.DialogXML = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "draw_table");
		glade_xml_signal_autoconnect (draw_box.DialogXML);

 		draw_box.visible = TRUE;
		draw_box.standalone = FALSE;
		draw_box.inmain = TRUE;
		draw_box.dialog_name = " Draw context ";
		draw_box.icon = SODIPODI_GLADEDIR "/toolbox_draw.xpm";
		sp_toolbox_create_widgets (&draw_box);
		draw_box.BoxTable = glade_xml_get_widget (draw_box.DialogXML, "draw_table");
		draw_select = glade_xml_get_widget (draw_box.DialogXML, "draw_select");
		draw_node = glade_xml_get_widget (draw_box.DialogXML, "draw_node");
		draw_zoom = glade_xml_get_widget (draw_box.DialogXML, "draw_zoom");
		draw_text = glade_xml_get_widget (draw_box.DialogXML, "draw_text");
		draw_rect = glade_xml_get_widget (draw_box.DialogXML, "draw_rect");
		draw_ellipse = glade_xml_get_widget (draw_box.DialogXML, "draw_ellipse");
		draw_freehand = glade_xml_get_widget (draw_box.DialogXML, "draw_freehand");
		gtk_toggle_button_set_active ((GtkToggleButton *) draw_select, TRUE);
		sp_toolbox_expose (&draw_box);

		gtk_signal_connect_while_alive (GTK_OBJECT (SODIPODI), "set_eventcontext",
						GTK_SIGNAL_FUNC (sp_update_draw_toolbox), NULL,
						GTK_OBJECT (draw_box.BoxTable));

		// zoom toolbox
		zoom_box.DialogXML = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "zoom_table");
		glade_xml_signal_autoconnect (zoom_box.DialogXML);

		zoom_box.visible = FALSE;
		zoom_box.standalone = FALSE;
		zoom_box.inmain = TRUE;
		zoom_box.dialog_name = " Zoom ";
		zoom_box.icon = SODIPODI_GLADEDIR "/toolbox_zoom.xpm";

		sp_toolbox_create_widgets (&zoom_box);
		zoom_box.BoxTable = glade_xml_get_widget (zoom_box.DialogXML, "zoom_table");
		zoom_any = glade_xml_get_widget (zoom_box.DialogXML, "zoom_any_entry");
		sp_toolbox_expose (&zoom_box);
		// node toolbox
		node_box.DialogXML = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "node_table");
		glade_xml_signal_autoconnect (node_box.DialogXML);

		node_box.visible = FALSE;
		node_box.standalone = FALSE;
		node_box.inmain = TRUE;
		node_box.dialog_name = " Node edit";
		node_box.icon = SODIPODI_GLADEDIR "/toolbox_node.xpm";

		sp_toolbox_create_widgets (&node_box);
		node_box.BoxTable = glade_xml_get_widget (node_box.DialogXML, "node_table");
		sp_toolbox_expose (&node_box);


		gtk_widget_show (main_toolbox_dialog);
	}
}


void sp_toolbox_create_widgets (SPToolBox * toolbox) {
  GtkWidget * vbox, * hbox;
  GtkWidget * gnpixmap;
  GtkWidget * sep, * label;

  // dialog
  toolbox->Dialog = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title ((GtkWindow *) toolbox->Dialog, toolbox->dialog_name);
  gtk_window_set_policy ((GtkWindow *) toolbox->Dialog,0,0,0);
  // main_vbox
  toolbox->MainVBox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start ((GtkBox *) main_vbox, toolbox->MainVBox, TRUE, TRUE, 0);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start ((GtkBox *) toolbox->MainVBox, hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);
  // seperate button
  toolbox->Seperate = gtk_button_new ();
  gtk_button_set_relief ((GtkButton *) toolbox->Seperate, GTK_RELIEF_NONE);
  gtk_box_pack_end ((GtkBox *) hbox, toolbox->Seperate, FALSE, FALSE, 0);
  gtk_widget_show (toolbox->Seperate);

  gnpixmap = gnome_pixmap_new_from_file (SODIPODI_GLADEDIR "/seperate_tool.xpm");
  gtk_container_add ((GtkContainer *) toolbox->Seperate, gnpixmap);
  gtk_widget_show (gnpixmap);
  // hide button
  toolbox->Hide = gtk_button_new  ();
  gtk_button_set_relief ((GtkButton *) toolbox->Hide, GTK_RELIEF_NONE);
  gtk_box_pack_start ((GtkBox *) hbox, toolbox->Hide, TRUE, TRUE, 0);
  gtk_widget_show (toolbox->Hide);

  hbox = gtk_hbox_new (FALSE,0);
  gtk_container_add ((GtkContainer *) toolbox->Hide, hbox);
  gtk_widget_show (hbox);
  gnpixmap = gnome_pixmap_new_from_file (toolbox->icon);
  gtk_box_pack_start ((GtkBox *) hbox, gnpixmap, FALSE, FALSE, 0);
  gtk_widget_show (gnpixmap);
  label = gtk_label_new (toolbox->dialog_name);
  gtk_box_pack_start ((GtkBox *) hbox, label, FALSE, FALSE, 0);
  gtk_widget_show (label);  
  sep = gtk_hseparator_new ();
  gtk_box_pack_start ((GtkBox *) hbox, sep, TRUE, TRUE, 0);
  gtk_widget_show (sep);  
  toolbox->Hide->allocation.height = 12;
  gtk_object_set(GTK_OBJECT(toolbox->Hide),  "GtkWidget::height", (gulong) 16, NULL);
  // dialog vbox
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add ((GtkContainer *) toolbox->Dialog, vbox);
  toolbox->DialogVBox = gtk_vbox_new (FALSE, 0);
  gtk_container_add ((GtkContainer *) vbox, toolbox->DialogVBox);
  gtk_widget_show (vbox);
  gtk_widget_show (toolbox->DialogVBox);
  // close button
  toolbox->Close = gnome_stock_button ("Button_Close");
  gtk_container_add ((GtkContainer *) vbox, toolbox->Close);
  gtk_widget_show (toolbox->Close);
  // signals
  gtk_signal_connect (GTK_OBJECT (toolbox->Hide), "clicked", GTK_SIGNAL_FUNC (toolbox_toggle_main), toolbox);
  gtk_signal_connect (GTK_OBJECT (toolbox->Seperate), "clicked", GTK_SIGNAL_FUNC (toolbox_toggle_seperate), toolbox);
  gtk_signal_connect (GTK_OBJECT (toolbox->Close), "clicked", GTK_SIGNAL_FUNC (toolbox_toggle_seperate), toolbox);
  gtk_signal_connect (GTK_OBJECT (toolbox->Dialog), "delete_event", GTK_SIGNAL_FUNC (toolbox_delete), toolbox);

}

void sp_toolbox_expose (SPToolBox * toolbox) {
  if (toolbox->inmain) gtk_widget_show (toolbox->MainVBox);
  if (toolbox->standalone) {
    gtk_box_pack_start ((GtkBox *) toolbox->DialogVBox, toolbox->BoxTable, TRUE, TRUE, 0);
    if (toolbox->visible) gtk_widget_show (toolbox->Dialog);
  } else {
    gtk_box_pack_end ((GtkBox *) toolbox->MainVBox, toolbox->BoxTable, TRUE, TRUE, 0);
    if (toolbox->visible) gtk_widget_show (toolbox->BoxTable);
  }
}


gint
sp_maintoolbox_close (GtkWidget * widget, GdkEventAny * event, gpointer data)
{
	sodipodi_unref ();

	return FALSE;
#if 0
  // we need save changes and stuff
  	if (!GTK_IS_WIDGET (quit_dialog)) {
  		quit_xml = glade_xml_new (SODIPODI_GLADEDIR "/toolbox.glade", "quit");
  		if (quit_xml == NULL) return;
		glade_xml_signal_autoconnect (quit_xml);
		quit_dialog = glade_xml_get_widget (quit_xml, "quit");
		if (quit_dialog == NULL) return;
	}else {
	  if (!GTK_WIDGET_VISIBLE (quit_dialog)) gtk_widget_show (quit_dialog);
	}
#endif
}

void quit_yep () {	gtk_main_quit();  }

void quit_nope () {  if (GTK_WIDGET_VISIBLE (quit_dialog)) gtk_widget_hide (quit_dialog); }

void toolbox_toggle_main (GtkWidget * widget, SPToolBox * toolbox) {
  if (!GTK_IS_WIDGET (toolbox->BoxTable)) g_print("table weg  ");
  if ((*toolbox).standalone) {
      (*toolbox).visible = TRUE;
      (*toolbox).standalone = FALSE;
      if (GTK_WIDGET_VISIBLE (toolbox->Dialog)) gtk_widget_hide (toolbox->Dialog);
      gtk_widget_reparent (toolbox->BoxTable, toolbox->MainVBox);
      if (!GTK_WIDGET_VISIBLE (toolbox->BoxTable)) gtk_widget_show (toolbox->BoxTable);
    }  else {
    if ((*toolbox).visible) {
      (*toolbox).visible = FALSE;
      if (GTK_WIDGET_VISIBLE (toolbox->BoxTable)) gtk_widget_hide (toolbox->BoxTable);
    } else {
      (*toolbox).visible = TRUE;
      if (!GTK_WIDGET_VISIBLE (toolbox->BoxTable)) gtk_widget_show (toolbox->BoxTable);
    }      
  }
}

void toolbox_close (GtkWidget * widget, SPToolBox * toolbox) {
  (*toolbox).visible = FALSE;
  if (GTK_WIDGET_VISIBLE (toolbox->Dialog)) gtk_widget_hide (toolbox->Dialog);
}

void toolbox_delete (GtkWidget * widget, SPToolBox * toolbox) {
  gtk_widget_hide (widget);
  (*toolbox).visible = FALSE;
}

void maintoolbox_destroy (GtkWidget * widget, SPToolBox * toolbox) {
  g_print ("goodbye !! ... ");
}

void toolbox_toggle_seperate (GtkWidget * widget, SPToolBox * toolbox) {
  if ((*toolbox).standalone) {
    if (GTK_WIDGET_VISIBLE (toolbox->Dialog)) {
      (*toolbox).visible = FALSE;
      gtk_widget_hide (toolbox->Dialog);
    } else {
      (*toolbox).visible = TRUE;
      if (!GTK_WIDGET_VISIBLE (toolbox->Dialog)) gtk_widget_show (toolbox->Dialog);
    }
  } else {
    (*toolbox).standalone = TRUE;
    (*toolbox).visible = TRUE;
    gtk_widget_reparent (toolbox->BoxTable, toolbox->DialogVBox);
    if (!GTK_WIDGET_VISIBLE (toolbox->BoxTable)) gtk_widget_show (toolbox->BoxTable);
    if (!GTK_WIDGET_VISIBLE (toolbox->Dialog)) gtk_widget_show (toolbox->Dialog);
  }
    
  g_print ("seperate ");
}

static void 
sp_update_draw_toolbox (Sodipodi * sodipodi, SPEventContext * eventcontext, gpointer data) {
	SPDesktop * desktop;

	if (eventcontext == NULL) return;

	desktop = eventcontext->desktop;

  if (GTK_OBJECT_TYPE(desktop->event_context) == SP_TYPE_SELECT_CONTEXT) 
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_select), TRUE);
  else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_select), FALSE);
  if (GTK_OBJECT_TYPE(desktop->event_context) == SP_TYPE_NODE_CONTEXT) 
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_node), TRUE);
  else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_node), FALSE);
  if (GTK_OBJECT_TYPE(desktop->event_context) == SP_TYPE_ZOOM_CONTEXT) 
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_zoom), TRUE);
  else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_zoom), FALSE);
  if (GTK_OBJECT_TYPE(desktop->event_context) == SP_TYPE_RECT_CONTEXT) 
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_rect), TRUE);
  else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_rect), FALSE);
  if (GTK_OBJECT_TYPE(desktop->event_context) == SP_TYPE_ELLIPSE_CONTEXT) 
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_ellipse), TRUE);
  else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_ellipse), FALSE);
  if (GTK_OBJECT_TYPE(desktop->event_context) == SP_TYPE_DRAW_CONTEXT) 
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_freehand), TRUE);
  else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_freehand), FALSE);
  if (GTK_OBJECT_TYPE(desktop->event_context) == SP_TYPE_TEXT_CONTEXT) 
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_text), TRUE);
  else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (draw_text), FALSE);
}


/* 
 * object toolbox
 */

void
object_flip (GtkWidget * widget, GdkEventButton * event) {
  SPDesktop * desktop;
  SPSelection * selection;
  SPItem * item;
  GSList * l, * l2;

  //right click
  if (event->button == 3) {
    if (object_flip_mode == FLIP_HOR) {
      object_flip_mode = FLIP_VER;
      gtk_widget_hide (fh_pixmap);
      gtk_widget_show (fv_pixmap);
    } else {
      object_flip_mode = FLIP_HOR;
      gtk_widget_hide (fv_pixmap);
      gtk_widget_show (fh_pixmap);
    };
  };

  // left click
  if (event->button == 1) {
    desktop = SP_ACTIVE_DESKTOP;
    if (!SP_IS_DESKTOP(desktop)) return;
    selection = SP_DT_SELECTION(desktop);
    if sp_selection_is_empty(selection) return;
    l = selection->items;  
    for (l2 = l; l2 != NULL; l2 = l2-> next) {
      item = SP_ITEM (l2->data);
      if (object_flip_mode == FLIP_HOR) sp_item_scale_rel (item,-1,1);
      else sp_item_scale_rel (item,1,-1);
    }

    sp_selection_changed (selection);
    sp_document_done (SP_DT_DOCUMENT (desktop));
  }

}

void object_rotate_90 () {
  SPDesktop * desktop;
  SPSelection * selection;
  SPItem * item;
  GSList * l, * l2;

  desktop = SP_ACTIVE_DESKTOP;
  if (!SP_IS_DESKTOP(desktop)) return;
  selection = SP_DT_SELECTION(desktop);
  if sp_selection_is_empty(selection) return;
  l = selection->items;  
  for (l2 = l; l2 != NULL; l2 = l2-> next) {
    item = SP_ITEM (l2->data);
    sp_item_rotate_rel (item,-90);
  }

  sp_selection_changed (selection);
  sp_document_done (SP_DT_DOCUMENT (desktop));

}
